// Adapted from: https://github.com/wong2/chat-gpt-google-extension/blob/main/background/index.mjs
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { createParser } from 'eventsource-parser';
import { v4 as uuidv4 } from 'uuid';
import ExpiryMap from 'expiry-map';
import fetch from 'node-fetch';
var KEY_ACCESS_TOKEN = 'accessToken';
var cache = new ExpiryMap(10 * 1000);
export function refreshAccessToken(sessionToken) {
    return __awaiter(this, void 0, void 0, function () {
        var resp;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (cache.get(KEY_ACCESS_TOKEN)) {
                        return [2 /*return*/, cache.get(KEY_ACCESS_TOKEN)];
                    }
                    return [4 /*yield*/, fetch('https://chat.openai.com/api/auth/session', {
                            headers: {
                                cookie: '__Secure-next-auth.session-token=' + sessionToken
                            }
                        })
                            .then(function (r) { return r.json(); })["catch"](function () { return ({}); })];
                case 1:
                    resp = _a.sent();
                    if (!resp.accessToken) {
                        throw new Error('Unathorized');
                    }
                    cache.set(KEY_ACCESS_TOKEN, resp.accessToken);
                    return [2 /*return*/, resp.accessToken];
            }
        });
    });
}
var ChatGPTClient = /** @class */ (function () {
    function ChatGPTClient(config, converstationId) {
        if (converstationId === void 0) { converstationId = uuidv4(); }
        this.config = config;
        this.converstationId = converstationId;
    }
    ChatGPTClient.prototype.ensureAuth = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, refreshAccessToken(this.config.sessionToken)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    ChatGPTClient.prototype.getAnswer = function (question) {
        return __awaiter(this, void 0, void 0, function () {
            var accessToken, response;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, refreshAccessToken(this.config.sessionToken)];
                    case 1:
                        accessToken = _a.sent();
                        response = '';
                        return [2 /*return*/, new Promise(function (resolve, reject) {
                                fetchSSE('https://chat.openai.com/backend-api/conversation', {
                                    method: 'POST',
                                    headers: {
                                        'Content-Type': 'application/json',
                                        Authorization: "Bearer ".concat(accessToken)
                                    },
                                    body: JSON.stringify({
                                        action: 'next',
                                        messages: [
                                            {
                                                id: uuidv4(),
                                                role: 'user',
                                                content: {
                                                    content_type: 'text',
                                                    parts: [question]
                                                }
                                            },
                                        ],
                                        model: 'text-davinci-002-render',
                                        parent_message_id: _this.converstationId
                                    }),
                                    onMessage: function (message) {
                                        var _a, _b, _c;
                                        if (message === '[DONE]') {
                                            return resolve(response);
                                        }
                                        var data = JSON.parse(message);
                                        var text = (_c = (_b = (_a = data.message) === null || _a === void 0 ? void 0 : _a.content) === null || _b === void 0 ? void 0 : _b.parts) === null || _c === void 0 ? void 0 : _c[0];
                                        if (text) {
                                            response = text;
                                        }
                                    }
                                })["catch"](reject);
                            })];
                }
            });
        });
    };
    return ChatGPTClient;
}());
export { ChatGPTClient };
function fetchSSE(resource, options) {
    return __awaiter(this, void 0, void 0, function () {
        var onMessage, fetchOptions, resp, parser;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    onMessage = options.onMessage, fetchOptions = __rest(options, ["onMessage"]);
                    return [4 /*yield*/, fetch(resource, fetchOptions)];
                case 1:
                    resp = _a.sent();
                    parser = createParser(function (event) {
                        if (event.type === 'event') {
                            onMessage(event.data);
                        }
                    });
                    resp.body.on('readable', function () {
                        var chunk;
                        while (null !== (chunk = resp.body.read())) {
                            parser.feed(chunk.toString());
                        }
                    });
                    return [2 /*return*/];
            }
        });
    });
}
